
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '32.8.0',
    'branch_nick': 'release-32-LTS',
    'revision_id': 'd8ff43713d63f1a6295fe21df8321f62b429a568',
    'revno': 12656
}
